#
# PERL module for connecting to XUCP protocol server
#

package XUCP;

use Exporter ();
@ISA = qw(Exporter);

use integer;
use strict;
use IO::Handle '_IOLBF';
use IO::Socket;
use Digest::MD5;

sub new {
    my($host, $port) = @_;
    my($sock, $in, $out);

    $sock = IO::Socket::INET->new(PeerAddr => $host,
				  PeerPort => $port,
				  Proto    => 'tcp');
    $in  = new IO::Handle->fdopen($sock, "r");
    $out = new IO::Handle->fdopen($sock, "w");
    $sock->close(); undef $sock;

    my($self);
    $self = {
	in   => $in,
	out  => $out,
	seq  => 0,
    };
    bless $self;

    my $line = $self->{in}->getline();
    my @line = split(/[ \t\n\r]/, $line);

    # printf("input line: '%s'\n", join(':', @line));

    if ($line[1] eq 'OK') {
	$self->{salt} = $line[2];
	return $self;
    }

    undef $self;
    return undef;
};

sub setdebug {
    my $self = shift;
    my ($val) = @_;

    if ($val != 0) {
	$self->{debug} = 1;
    } else {
	undef $self->{debug};
    }
};

#sub DESTROY {
#    my $self = shift;
#    undef $self->{in};
#    undef $self->{out};
#    undef $self;
#};


sub sendcmd {
    my $self = shift;
    my ($cmd) = @_;
    my $line;

    if ($self->{debug}) {
	printf("sendcmd: cmd='%s'\n", $cmd);
    }

    $line = sprintf("%d %s\r\n", $self->{seq}, $cmd);
    $self->{seq} += 1;

    $self->{out}->write($line, length($line));
    $self->{out}->flush();

    $line = $self->{in}->getline();
    my @line = split(/[ \t\n\r]/, $line);
    $self->{resp} = join(' ', @line);

    if ($self->{debug}) {
	printf("result: %s\n", $self->{resp});
    }
};


#
#  XUCP::login(uname, passwd)
#

sub login {
    my $self = shift;
    my ($login, $passwd) = @_;

    my $authenticator = Digest::MD5::md5_hex($self->{salt} . $passwd);

    my $cmd = sprintf("login %s %s", $login, $authenticator);
    return $self->sendcmd($cmd);
};

#
#  XUCP::bye()
#

sub bye {
    my $self = shift;

    my $cmd = sprintf("bye");
    return $self->sendcmd($cmd);
};


#
# XUCP->gsmfrom("fromnumber");
#

sub gsmfrom {
    my $self = shift;
    my ($gsmfrom) = @_;

    my $cmd = sprintf("meta gsmfrom=%s", $gsmfrom);
    return $self->sendcmd($cmd);
};


#
# XUCP->rcpt("channel", "number");
#

sub rcpt {
    my $self = shift;
    my ($channel,$number) = @_;

    my $cmd = sprintf("rcpt %s %s", $channel, $number);
    return $self->sendcmd($cmd);
};

#
# XUCP->bidref("text string")
#

sub bidref {
    my $self = shift;
    my ($ref) = @_;

    my $cmd = sprintf("bidref %s", $ref);
    return $self->sendcmd($cmd);
}

#
# XUCP->text("text string")
#

sub text {
    my $self = shift;
    my ($text) = @_;

    my $cmd = sprintf("text %s", $text);
    return $self->sendcmd($cmd);
}

#
# XUCP->gsmexpiry("DDMMYYhhmm")
#

sub gsmexpiry {
    my $self = shift;
    my ($gsmexpiry) = @_;

    my $cmd = sprintf("meta gsmexpiry=%s", $gsmexpiry);
    return $self->sendcmd($cmd);
}
